Exception: GitCache::Error

Inherits:
StandardError
  • Object
show all
Defined in:
lib/git_cache/error.rb

Overview

GitCache encountered a failure

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(message, result) ⇒ Error

Create a GitCache::Error.

Parameters:

  • message (String)

    The error message

  • result (::ExecService::Result)

    The result of a git command execution, or nil if this error was not due to a git command error.



16
17
18
19
# File 'lib/git_cache/error.rb', line 16

def initialize(message, result)
  super(message)
  @exec_result = result
end

Instance Attribute Details

#exec_result::ExecService::Result (readonly)

Returns The result of a git command execution, or nil if this error was not due to a git command error.

Returns:

  • (::ExecService::Result)

    The result of a git command execution, or nil if this error was not due to a git command error.



26
27
28
# File 'lib/git_cache/error.rb', line 26

def exec_result
  @exec_result
end